// This file was generated by qlalr - DO NOT EDIT!
#ifndef REP_GRAMMAR_P_H
#define REP_GRAMMAR_P_H

class rep_grammar
{
public:
    enum VariousConstants {
        EOF_SYMBOL = 0,
        Token_childrep = 12,
        Token_class = 2,
        Token_comma = 19,
        Token_comment = 21,
        Token_enum = 6,
        Token_equals = 20,
        Token_flag = 5,
        Token_mcomment = 22,
        Token_model = 11,
        Token_newline = 24,
        Token_passbyqual = 14,
        Token_pod = 3,
        Token_pod2 = 4,
        Token_preprocessor_directive = 23,
        Token_prop = 7,
        Token_qualifier = 13,
        Token_semicolon = 1,
        Token_signal = 9,
        Token_slot = 10,
        Token_start = 17,
        Token_stop = 18,
        Token_symbol = 15,
        Token_tstart = 25,
        Token_tstop = 26,
        Token_use_enum = 8,
        Token_value = 16,

        ACCEPT_STATE = 176,
        RULE_COUNT = 159,
        STATE_COUNT = 182,
        TERMINAL_COUNT = 27,
        NON_TERMINAL_COUNT = 60,

        GOTO_INDEX_OFFSET = 182,
        GOTO_INFO_OFFSET = 368,
        GOTO_CHECK_OFFSET = 368
    };

    static const char *const     spell[];
    static const short             lhs[];
    static const short             rhs[];

#ifndef QLALR_NO_REP_GRAMMAR_DEBUG_INFO
    static const int     rule_index[];
    static const int      rule_info[];
#endif // QLALR_NO_REP_GRAMMAR_DEBUG_INFO

    static const short    goto_default[];
    static const short  action_default[];
    static const short    action_index[];
    static const short     action_info[];
    static const short    action_check[];

    static inline int nt_action (int state, int nt)
    {
        const int yyn = action_index [GOTO_INDEX_OFFSET + state] + nt;
        if (yyn < 0 || action_check [GOTO_CHECK_OFFSET + yyn] != nt)
            return goto_default [nt];

        return action_info [GOTO_INFO_OFFSET + yyn];
    }

    static inline int t_action (int state, int token)
    {
        const int yyn = action_index [state] + token;

        if (yyn < 0 || action_check [yyn] != token)
            return - action_default [state];

        return action_info [yyn];
    }
};


#endif // REP_GRAMMAR_P_H

